@@ -1,5 +1,5 @@
-// SPDX-License-Identifier: GPL-2.0
-/* Copyright(c) 2007 - 2020 Intel Corporation. */
+/* SPDX-License-Identifier: @SPDX@ */
+/* Copyright(c) 2007 - 2023 Intel Corporation. */
 
 #include "igb.h"
 #include "kcompat.h"
@@ -1094,7 +1094,7 @@ out:
 
 /*****************************************************************************/
 #if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29) )
-static void __kc_pci_set_master(struct pci_dev *pdev, bool enable)
+static void __kc_pci_set_main(struct pci_dev *pdev, bool enable)
 {
 	u16 old_cmd, cmd;
 
@@ -1104,7 +1104,7 @@ static void __kc_pci_set_master(struct pci_dev *pdev, bool enable)
 	else
 		cmd = old_cmd & ~PCI_COMMAND_MASTER;
 	if (cmd != old_cmd) {
-		dev_dbg(pci_dev_to_dev(pdev), "%s bus mastering\n",
+		dev_dbg(pci_dev_to_dev(pdev), "%s bus DMA control\n",
 			enable ? "enabling" : "disabling");
 		pci_write_config_word(pdev, PCI_COMMAND, cmd);
 	}
@@ -1113,9 +1113,9 @@ static void __kc_pci_set_master(struct pci_dev *pdev, bool enable)
 #endif
 }
 
-void _kc_pci_clear_master(struct pci_dev *dev)
+void _kc_pci_clear_main(struct pci_dev *dev)
 {
-	__kc_pci_set_master(dev, false);
+	__kc_pci_set_main(dev, false);
 }
 #endif /* < 2.6.29 */
 
@@ -1579,7 +1579,6 @@ static const unsigned char __maybe_unused pcie_link_speed[] = {
 int __kc_pcie_get_minimum_link(struct pci_dev *dev, enum pci_bus_speed *speed,
 			       enum pcie_link_width *width)
 {
-	int ret;
 
 	*speed = PCI_SPEED_UNKNOWN;
 	*width = PCIE_LNK_WIDTH_UNKNOWN;
@@ -1588,8 +1587,8 @@ int __kc_pcie_get_minimum_link(struct pci_dev *dev, enum pci_bus_speed *speed,
 		u16 lnksta;
 		enum pci_bus_speed next_speed;
 		enum pcie_link_width next_width;
+		int ret = pcie_capability_read_word(dev, PCI_EXP_LNKSTA, &lnksta);
 
-		ret = pcie_capability_read_word(dev, PCI_EXP_LNKSTA, &lnksta);
 		if (ret)
 			return ret;
 
@@ -1701,7 +1700,6 @@ int __kc_ipv6_find_hdr(const struct sk_buff *skb, unsigned int *offset,
 {
 	unsigned int start = skb_network_offset(skb) + sizeof(struct ipv6hdr);
 	u8 nexthdr = ipv6_hdr(skb)->nexthdr;
-	unsigned int len;
 	bool found;
 
 #define __KC_IP6_FH_F_FRAG	BIT(0)
@@ -1722,7 +1720,6 @@ int __kc_ipv6_find_hdr(const struct sk_buff *skb, unsigned int *offset,
 		start = *offset + sizeof(struct ipv6hdr);
 		nexthdr = ip6->nexthdr;
 	}
-	len = skb->len - start;
 
 	do {
 		struct ipv6_opt_hdr _hdr, *hp;
@@ -1787,7 +1784,6 @@ int __kc_ipv6_find_hdr(const struct sk_buff *skb, unsigned int *offset,
 
 		if (!found) {
 			nexthdr = hp->nexthdr;
-			len -= hdrlen;
 			start += hdrlen;
 		}
 	} while (!found);
@@ -2274,11 +2270,11 @@ unsigned int _kc_cpumask_local_spread(unsigned int i, int node)
  * and this function is in no way similar to skb_flow_dissect_flow_keys(). An
  * example use can be found in the ice driver, specifically ice_arfs.c.
  *
- * This function is treated as a whitelist of supported fields the SKB can
+ * This function is treated as a allowlist of supported fields the SKB can
  * parse. If new functionality is added make sure to keep this format (i.e. only
  * check for fields that are explicity wanted).
  *
- * Current whitelist:
+ * Current allowlist:
  *
  * TCPv4, TCPv6, UDPv4, UDPv6
  *
@@ -2394,6 +2390,47 @@ int _kc_eth_platform_get_mac_address(struct device *dev __maybe_unused,
 #endif /* !(RHEL_RELEASE >= 7.3) */
 #endif /* < 4.5.0 */
 
+/*****************************************************************************/
+#if (LINUX_VERSION_CODE < KERNEL_VERSION(4,6,0))
+int _kc_kstrtobool(const char *s, bool *res)
+{
+	if (!s)
+		return -EINVAL;
+
+	switch (s[0]) {
+	case 'y':
+	case 'Y':
+	case '1':
+		*res = true;
+		return 0;
+	case 'n':
+	case 'N':
+	case '0':
+		*res = false;
+		return 0;
+	case 'o':
+	case 'O':
+		switch (s[1]) {
+		case 'n':
+		case 'N':
+			*res = true;
+			return 0;
+		case 'f':
+		case 'F':
+			*res = false;
+			return 0;
+		default:
+			break;
+		}
+		break;
+	default:
+		break;
+	}
+
+	return -EINVAL;
+}
+#endif /* < 4.6.0 */
+
 /*****************************************************************************/
 #if ((LINUX_VERSION_CODE < KERNEL_VERSION(4,14,0)) || \
      (SLE_VERSION_CODE && (SLE_VERSION_CODE <= SLE_VERSION(12,3,0))) || \
@@ -2428,6 +2465,10 @@ const char *_kc_phy_speed_to_str(int speed)
 #ifdef SPEED_100000
 	case SPEED_100000:
 		return "100Gbps";
+#endif
+#ifdef SPEED_200000
+	case SPEED_200000:
+		return "200Gbps";
 #endif
 	case SPEED_UNKNOWN:
 		return "Unknown";
@@ -2454,38 +2495,6 @@ void _kc_ethtool_intersect_link_masks(struct ethtool_link_ksettings *dst,
 }
 #endif /* 4.15.0 */
 
-/*****************************************************************************/
-#if (LINUX_VERSION_CODE < KERNEL_VERSION(4,16,0))
-#if !(RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(8,0)) && \
-     !(SLE_VERSION_CODE >= SLE_VERSION(12,5,0) && \
-       SLE_VERSION_CODE < SLE_VERSION(15,0,0) || \
-       SLE_VERSION_CODE >= SLE_VERSION(15,1,0))
-#if BITS_PER_LONG == 64
-/**
- * bitmap_from_arr32 - copy the contents of u32 array of bits to bitmap
- * @bitmap: array of unsigned longs, the destination bitmap
- * @buf: array of u32 (in host byte order), the source bitmap
- * @nbits: number of bits in @bitmap
- */
-void bitmap_from_arr32(unsigned long *bitmap, const u32 *buf, unsigned int nbits)
-{
-	unsigned int i, halfwords;
-
-	halfwords = DIV_ROUND_UP(nbits, 32);
-	for (i = 0; i < halfwords; i++) {
-		bitmap[i/2] = (unsigned long) buf[i];
-		if (++i < halfwords)
-			bitmap[i/2] |= ((unsigned long) buf[i]) << 32;
-	}
-
-	/* Clear tail bits in last word beyond nbits. */
-	if (nbits % BITS_PER_LONG)
-		bitmap[(halfwords - 1) / 2] &= BITMAP_LAST_WORD_MASK(nbits);
-}
-#endif /* BITS_PER_LONG == 64 */
-#endif /* !(RHEL >= 8.0) && !(SLES >= 12.5 && SLES < 15.0 || SLES >= 15.1) */
-#endif /* 4.16.0 */
-
 /*****************************************************************************/
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(4,17,0))
 /* PCIe link information */
@@ -2634,98 +2643,6 @@ void _kc_pcie_print_link_status(struct pci_dev *dev) {
 }
 #endif /* 4.17.0 */
 
-/*****************************************************************************/
-#if (LINUX_VERSION_CODE < KERNEL_VERSION(5,1,0))
-#if (RHEL_RELEASE_CODE && (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(8,1)))
-#define HAVE_NDO_FDB_ADD_EXTACK
-#else /* !RHEL || RHEL < 8.1 */
-#ifdef HAVE_TC_SETUP_CLSFLOWER
-#define FLOW_DISSECTOR_MATCH(__rule, __type, __out)				\
-	const struct flow_match *__m = &(__rule)->match;			\
-	struct flow_dissector *__d = (__m)->dissector;				\
-										\
-	(__out)->key = skb_flow_dissector_target(__d, __type, (__m)->key);	\
-	(__out)->mask = skb_flow_dissector_target(__d, __type, (__m)->mask);	\
-
-void flow_rule_match_basic(const struct flow_rule *rule,
-			   struct flow_match_basic *out)
-{
-	FLOW_DISSECTOR_MATCH(rule, FLOW_DISSECTOR_KEY_BASIC, out);
-}
-
-void flow_rule_match_control(const struct flow_rule *rule,
-			     struct flow_match_control *out)
-{
-	FLOW_DISSECTOR_MATCH(rule, FLOW_DISSECTOR_KEY_CONTROL, out);
-}
-
-void flow_rule_match_eth_addrs(const struct flow_rule *rule,
-			       struct flow_match_eth_addrs *out)
-{
-	FLOW_DISSECTOR_MATCH(rule, FLOW_DISSECTOR_KEY_ETH_ADDRS, out);
-}
-
-#ifdef HAVE_TC_FLOWER_ENC
-void flow_rule_match_enc_keyid(const struct flow_rule *rule,
-			       struct flow_match_enc_keyid *out)
-{
-	FLOW_DISSECTOR_MATCH(rule, FLOW_DISSECTOR_KEY_ENC_KEYID, out);
-}
-
-void flow_rule_match_enc_ports(const struct flow_rule *rule,
-			       struct flow_match_ports *out)
-{
-	FLOW_DISSECTOR_MATCH(rule, FLOW_DISSECTOR_KEY_ENC_PORTS, out);
-}
-
-void flow_rule_match_enc_control(const struct flow_rule *rule,
-				 struct flow_match_control *out)
-{
-	FLOW_DISSECTOR_MATCH(rule, FLOW_DISSECTOR_KEY_ENC_CONTROL, out);
-}
-
-void flow_rule_match_enc_ipv4_addrs(const struct flow_rule *rule,
-				    struct flow_match_ipv4_addrs *out)
-{
-	FLOW_DISSECTOR_MATCH(rule, FLOW_DISSECTOR_KEY_ENC_IPV4_ADDRS, out);
-}
-
-void flow_rule_match_enc_ipv6_addrs(const struct flow_rule *rule,
-				    struct flow_match_ipv6_addrs *out)
-{
-	FLOW_DISSECTOR_MATCH(rule, FLOW_DISSECTOR_KEY_ENC_IPV6_ADDRS, out);
-}
-#endif
-
-#ifndef HAVE_TC_FLOWER_VLAN_IN_TAGS
-void flow_rule_match_vlan(const struct flow_rule *rule,
-			  struct flow_match_vlan *out)
-{
-	FLOW_DISSECTOR_MATCH(rule, FLOW_DISSECTOR_KEY_VLAN, out);
-}
-#endif
-
-void flow_rule_match_ipv4_addrs(const struct flow_rule *rule,
-				struct flow_match_ipv4_addrs *out)
-{
-	FLOW_DISSECTOR_MATCH(rule, FLOW_DISSECTOR_KEY_IPV4_ADDRS, out);
-}
-
-void flow_rule_match_ipv6_addrs(const struct flow_rule *rule,
-				struct flow_match_ipv6_addrs *out)
-{
-	FLOW_DISSECTOR_MATCH(rule, FLOW_DISSECTOR_KEY_IPV6_ADDRS, out);
-}
-
-void flow_rule_match_ports(const struct flow_rule *rule,
-			   struct flow_match_ports *out)
-{
-	FLOW_DISSECTOR_MATCH(rule, FLOW_DISSECTOR_KEY_PORTS, out);
-}
-#endif /* HAVE_TC_SETUP_CLSFLOWER */
-#endif /* !RHEL || RHEL < 8.1 */
-#endif /* 5.1.0 */
-
 /*****************************************************************************/
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(5,3,0))
 #if (!(RHEL_RELEASE_CODE && (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(8,2))))
@@ -2786,3 +2703,135 @@ u64 _kc_pci_get_dsn(struct pci_dev *dev)
 	return dsn;
 }
 #endif /* 5.7.0 */
+
+#ifdef NEED_DEVM_KVASPRINTF
+char *devm_kvasprintf(struct device *dev, gfp_t gfp, const char *fmt,
+		      va_list ap)
+{
+	unsigned int len;
+	char *p;
+	va_list aq;
+
+	va_copy(aq, ap);
+	len = vsnprintf(NULL, 0, fmt, aq);
+	va_end(aq);
+
+	p = devm_kmalloc(dev, len + 1, gfp);
+	if (!p)
+		return NULL;
+
+	vsnprintf(p, len + 1, fmt, ap);
+
+	return p;
+}
+#endif /* NEED_DEVM_KVASPRINTF */
+
+#ifdef NEED_DEVM_KASPRINTF
+char *devm_kasprintf(struct device *dev, gfp_t gfp, const char *fmt, ...)
+{
+	va_list ap;
+	char *p;
+
+	va_start(ap, fmt);
+	p = devm_kvasprintf(dev, gfp, fmt, ap);
+	va_end(ap);
+
+	return p;
+}
+#endif /* NEED_DEVM_KASPRINTF */
+
+#ifdef NEED_PCI_IOV_VF_ID
+#ifdef CONFIG_PCI_IOV
+/*
+ * Below function needs to access pci_sriov offset and stride. Since
+ * pci_sriov structure is defined in drivers/pci/pci.h which can not
+ * be included as linux kernel header file, the structure definition
+ * is not globally visible.
+ * As a result, one copy of structure definition is added. Since the
+ * definition is a copy, you need to make sure the kernel you want
+ * to backport must have exactly the same pci_sriov definition as the
+ * copy, otherwise you'll access wrong field offset and value.
+ */
+
+/* Single Root I/O Virtualization */
+struct pci_sriov {
+	int             pos;            /* Capability position */
+	int             nres;           /* Number of resources */
+	u32             cap;            /* SR-IOV Capabilities */
+	u16             ctrl;           /* SR-IOV Control */
+	u16             total_VFs;      /* Total VFs associated with the PF */
+	u16             initial_VFs;    /* Initial VFs associated with the PF */
+	u16             num_VFs;        /* Number of VFs available */
+	u16             offset;         /* First VF Routing ID offset */
+	u16             stride;         /* Following VF stride */
+	u16             vf_device;      /* VF device ID */
+	u32             pgsz;           /* Page size for BAR alignment */
+	u8              link;           /* Function Dependency Link */
+	u8              max_VF_buses;   /* Max buses consumed by VFs */
+	u16             driver_max_VFs; /* Max num VFs driver supports */
+	struct pci_dev  *dev;           /* Lowest numbered PF */
+	struct pci_dev  *self;          /* This PF */
+	u32             cfg_size;       /* VF config space size */
+	u32             class;          /* VF device */
+	u8              hdr_type;       /* VF header type */
+	u16             subsystem_vendor; /* VF subsystem vendor */
+	u16             subsystem_device; /* VF subsystem device */
+	resource_size_t barsz[PCI_SRIOV_NUM_BARS];      /* VF BAR size */
+	bool            drivers_autoprobe; /* Auto probing of VFs by driver */
+};
+
+int _kc_pci_iov_vf_id(struct pci_dev *dev)
+{
+	struct pci_dev *pf;
+
+	if (!dev->is_virtfn)
+		return -EINVAL;
+
+	pf = pci_physfn(dev);
+	return (((dev->bus->number << 8) + dev->devfn) -
+		((pf->bus->number << 8) + pf->devfn + pf->sriov->offset)) /
+	       pf->sriov->stride;
+}
+#endif /* CONFIG_PCI_IOV */
+#endif /* NEED_PCI_IOV_VF_ID */
+
+#ifdef NEED_MUL_U64_U64_DIV_U64
+u64 mul_u64_u64_div_u64(u64 a, u64 b, u64 c)
+{
+	u64 res = 0, div, rem;
+	int shift;
+
+	/* can a * b overflow ? */
+	if (ilog2(a) + ilog2(b) > 62) {
+		/*
+		 * (b * a) / c is equal to
+		 *
+		 *      (b / c) * a +
+		 *      (b % c) * a / c
+		 *
+		 * if nothing overflows. Can the 1st multiplication
+		 * overflow? Yes, but we do not care: this can only
+		 * happen if the end result can't fit in u64 anyway.
+		 *
+		 * So the code below does
+		 *
+		 *      res = (b / c) * a;
+		 *      b = b % c;
+		 */
+		div = div64_u64_rem(b, c, &rem);
+		res = div * a;
+		b = rem;
+
+		shift = ilog2(a) + ilog2(b) - 62;
+		if (shift > 0) {
+			/* drop precision */
+			b >>= shift;
+			c >>= shift;
+			if (!c)
+				return res;
+		}
+	}
+
+	return res + div64_u64(a * b, c);
+}
+#endif /* NEED_MUL_U64_U64_DIV_U64 */
